// Craig S. Kaplan

// Use a public API for querying a database of
// stupid Donald Trump quotes (https://www.tronalddump.io/)
// to print out quotes matching a query string.

import controlP5.*;

// The ControlP5 factory and two UI elements: a text label to tell
// you what mode you're in, and a scrollable text area to show all
// the words found in that mode.
ControlP5 ui;
Textfield field;
Textarea area;

void setup()
{
  size( 400, 400 );

  // Set up the user interface
  ui = new ControlP5( this );

  field = ui.addTextfield( "" )
    .setPosition( 5, 5 )
    .setSize( 390, 40 )
    .setFont( createFont( "arial", 24 ) )
    .setColor( color( 255, 255, 255 ) )
    .setAutoClear( false );

  area = ui.addTextarea( "Words" )
    .setPosition( 5, 50 )
    .setSize( 390, 345 )
    .setFont( createFont( "arial", 20 ) )
    .setColor( color( 255, 255, 255 ) );

  System.setProperty("http.agent", "Chrome");
}

void draw()
{
  background( 50 );
}

void controlEvent( ControlEvent ce )
{
  try {
    if ( ce.isFrom( field ) ) {
      String txt = field.getText();

      // This is a bit messy, but it's just a way to make
      // sure that the query is encoded as a legal URL
      // (in case you include non-ASCII characters, or 
      // spaces, etc.)
      java.net.URI uri = new java.net.URI(
        "https", 
        "api.tronalddump.io", 
        "/search/quote", 
        "query=" + txt, 
        null );
      
      // At least, this is the URL we're after
      String request = uri.toASCIIString();
      
      // Use it to get the live JSON object from the server.
      JSONObject result = loadJSONObject( request );

      String res = "";

      if ( result.getInt( "count" ) > 0 ) {
        result = result.getJSONObject( "_embedded" );
        JSONArray arr = result.getJSONArray( "quotes" );

        for ( int idx = 0; idx < arr.size(); ++idx ) {
          JSONObject info = arr.getJSONObject( idx );

          String quote = info.getString( "value" );
          res = res + quote + "\n\n";
        }
      } else {
        res = "No quotes found.";
      }

      area.setText( res );
    }
  } 
  catch( Exception e ) {
    println( e );
    e.printStackTrace();
  }
}