// Craig S. Kaplan

// A demonstration of downloading a live RSS feed from the internet
// (in this case, the RSS feed for the podcast Radiolab) and displaying
// its information.  The RSS feed is converted in real time from its
// native XML format to JSON using a third-party service (rss2json.com).

import controlP5.*;

JSONObject feed;
int num_items;
int current;
PImage img;

ControlP5 ui;
Textarea title;
Textarea desc;

void setup()
{
  size( 570, 300 );

  // Set up the user interface
  ui = new ControlP5( this );

  title = ui.addTextarea( "Title" )
    .setPosition( 150, 5 )
    .setSize( 400, 50 )
    .setFont( createFont( "arial", 24 ) )
    .setColor( color( 255 ) );

  desc = ui.addTextarea( "Description" )
    .setPosition( 150, 60 )
    .setSize( 400, 230 )
    .setFont( createFont( "arial", 20 ) )
    .setColor( color( 200 ) );

  feed = loadJSONObject( "https://api.rss2json.com/v1/api.json?rss_url=http%3A%2F%2Ffeeds.wnyc.org%2Fradiolab%3Fformat%3Dxml" );
  num_items = feed.getJSONArray( "items" ).size();
  current = 1;
  setCurrent( 0 );
}

void draw()
{
  background( 50 );
  if ( img != null ) {
    image( img, 10, 10 );
  }
}

void setCurrent( int n )
{
  if ( current != n ) {
    current = n;
    JSONObject obj = feed.getJSONArray( "items" ).getJSONObject( n );
    title.setText( obj.getString( "title" ) );
    desc.setText( obj.getString( "description" ) );
    img = loadImage( obj.getString( "thumbnail" ) );
    img.resize( 130, 0 );
  }
}

void keyPressed()
{
  if ( keyCode == RIGHT ) {
    setCurrent( min( current + 1, num_items - 1 ) );
  } else if ( keyCode == LEFT ) {
    setCurrent( max( current - 1, 0 ) );
  }
}
