// Craig S. Kaplan

// An example of using JSON as a kind of "file format"
// for storing lots of big, complicated information.  In
// this case, the sketch is a first attempt at a slideshow
// program where the talk is stored as a JSON file.  There's
// a lot of code here, but none of it is particularly deep --
// it just takes a lot of lines of code to extract useful
// data from a JSON file and display it as a slide show.

JSONObject slides;
int current_slide;
int num_slides;

JSONObject getMaster( String name )
{
  JSONArray masters = slides.getJSONArray( "masters" );
  for ( int idx = 0; idx < masters.size(); ++idx ) {
    JSONObject m = masters.getJSONObject( idx );
    if ( m.getString( "name" ).equals( name ) ) {
      return m;
    }
  }

  return null;
}

JSONObject getBoxParams( JSONObject master, String boxname )
{
  JSONArray boxes = master.getJSONArray( "boxes" );
  for ( int idx = 0; idx < boxes.size(); ++idx ) {
    JSONObject m = boxes.getJSONObject( idx );
    if ( m.getString( "name" ).equals( boxname ) ) {
      return m;
    }
  }

  return null;
}

color getColour( JSONArray arr )
{
  return color( arr.getFloat( 0 ), arr.getFloat( 1 ), arr.getFloat( 2 ) );
}

int countLines( String txt )
{
  return splitTokens( txt, "\n" ).length;
}

void drawSlide( int num )
{
  JSONArray all = slides.getJSONArray( "slides" );
  JSONObject slide = all.getJSONObject( num );
  JSONObject master = getMaster( slide.getString( "master" ) );
  
  background( getColour( master.getJSONArray( "background" ) ) );
  
  JSONArray content = slide.getJSONArray( "content" );
  for( int idx = 0; idx < content.size(); ++idx ) {
    JSONObject bcon = content.getJSONObject( idx );
    JSONObject params = getBoxParams( master, bcon.getString( "box" ) );
   // println( bcon.getJSONObject( "box" ) );
    String txt = bcon.getString( "text" );
    int lines = countLines( txt );
    
    textSize( params.getFloat( "textsize" ) );
    String align = params.getString( "halign" );
    if( align.equals( "left" ) ) {
      textAlign( LEFT );
    } else if( align.equals( "centre" ) ) {
      textAlign( CENTER );
    }
    
    float yoffs = 0;
    align = params.getString( "valign" );
    if( align.equals( "top" ) ) {
      yoffs = textAscent();
    } else if( align.equals( "centre" ) ) {
      yoffs = textAscent() - (0.5*lines*(textAscent()+textDescent()));
    }
      
    fill( getColour( params.getJSONArray( "colour" ) ) );
    JSONArray loc = params.getJSONArray( "location" );
    text( txt, loc.getFloat( 0 ), loc.getFloat( 1 ) + yoffs );
  }
}

void setup()
{
  size( 800, 600 );

  slides = loadJSONObject( "my_show.json" );
  num_slides = slides.getJSONArray( "slides" ).size();
  
  textFont( createFont( "Gotham-Medium", 48 ) );
//  printArray( PFont.list() );
}

void draw()
{
  drawSlide( current_slide );
  noLoop();
}

void keyPressed()
{
  if( keyCode == LEFT ) {
    current_slide = max( 0, current_slide - 1 );
    loop();
  } else if( keyCode == RIGHT ) {
    current_slide = min( num_slides - 1, current_slide + 1 );
    loop();
  } else if( key == 'r' ) {
    slides = loadJSONObject( "my_show.json" );
    loop();
  }
}
